package main

import (
	"log"
	"net/http"
	"os"
	"time"

	b "streifling.com/jason/cpolis/cmd/backend"
	c "streifling.com/jason/cpolis/cmd/calls"
	f "streifling.com/jason/cpolis/cmd/frontend"
)

func main() {
	config, err := b.HandleConfig()
	if err != nil {
		log.Fatalln(err)
	}

	logFile, err := os.OpenFile(config.LogFile, os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0644)
	if err != nil {
		log.Fatalln(err)
	}
	defer logFile.Close()
	log.SetOutput(logFile)

	db, err := b.OpenDB(config)
	if err != nil {
		log.Fatalln(err)
	}
	defer db.Close()

	sessions, sessionExpiryChan := f.StartSessions()
	defer close(sessionExpiryChan)

	index, err := b.InitIndex(config)
	if err != nil {
		log.Fatalln(err)
	}

	go func(c *b.Config, db *b.DB) {
		for {
			if err = b.CleanUpImages(c, db); err != nil {
				log.Println(err)
			}
			time.Sleep(time.Hour * 24)
		}
	}(config, db)

	mux := http.NewServeMux()
	mux.Handle("/web/static/", http.StripPrefix("/web/static/",
		http.FileServer(http.Dir(config.WebDir+"/static/"))))
	mux.HandleFunc("/", f.HomePage(config, db, sessions))

	mux.HandleFunc("GET /article/allow-edit/{id}", f.AllowEditArticle(config, db, sessions))
	mux.HandleFunc("GET /article/all-published/review-edit", f.ShowPublishedArticles(config, db, sessions, "review-edit"))
	mux.HandleFunc("GET /article/all-published/delete", f.ShowPublishedArticles(config, db, sessions, "review-delete"))
	mux.HandleFunc("GET /article/all-rejected", f.ShowRejectedArticles(config, db, sessions))
	mux.HandleFunc("GET /article/all-unpublished-unrejected-and-published-rejected", f.ShowUnpublishedUnrejectedAndPublishedRejectedArticles(config, db, sessions))
	mux.HandleFunc("GET /article/delete/{id}", f.DeleteArticle(config, db, sessions))
	mux.HandleFunc("GET /article/edit/{id}", f.EditArticle(config, db, sessions))
	mux.HandleFunc("GET /article/publish/{id}", f.PublishArticle(config, db, sessions, index))
	mux.HandleFunc("GET /article/query/{query}", c.QueryArticles(config, index))
	mux.HandleFunc("GET /article/reject/{id}", f.RejectArticle(config, db, sessions))
	mux.HandleFunc("GET /article/review-delete/{id}", f.ReviewArticle(config, db, sessions, "delete", "Artikel löschen", "Löschen"))
	mux.HandleFunc("GET /article/review-edit/{id}", f.ReviewArticle(config, db, sessions, "allow-edit", "Artikel bearbeiten", "Bearbeiten erlauben"))
	mux.HandleFunc("GET /article/review-rejected/{id}", f.ReviewRejectedArticle(config, db, sessions))
	mux.HandleFunc("GET /article/review-unpublished/{id}", f.ReviewArticle(config, db, sessions, "publish", "Artikel veröffentlichen", "Veröffentlichen"))
	mux.HandleFunc("GET /article/serve/{uuid}", c.ServeArticle(config, db))
	mux.HandleFunc("GET /article/serve/{uuid}/clicks", c.ServeClicks(db))
	mux.HandleFunc("GET /article/write", f.WriteArticle(config, db, sessions))
	mux.HandleFunc("GET /atom/serve", c.ServeAtomFeed(config))
	mux.HandleFunc("GET /hub", f.ShowHub(config, db, sessions))
	mux.HandleFunc("GET /image/serve/{pic}", c.ServeImage(config, sessions))
	mux.HandleFunc("GET /issue/this", f.ShowCurrentIssue(config, db, sessions))
	mux.HandleFunc("GET /logout", f.Logout(config, sessions))
	mux.HandleFunc("GET /pdf/get-list", c.ServePDFList(config))
	mux.HandleFunc("GET /pdf/serve/{id}", c.ServePDF(config))
	mux.HandleFunc("GET /tag/create", f.CreateTag(config, sessions))
	mux.HandleFunc("GET /user/create", f.CreateUser(config, sessions))
	mux.HandleFunc("GET /user/delete/{id}", f.DeleteUser(config, db, sessions))
	mux.HandleFunc("GET /user/edit/{id}", f.EditUser(config, db, sessions))
	mux.HandleFunc("GET /user/edit/self", f.EditSelf(config, db, sessions))
	mux.HandleFunc("GET /user/show-all/delete", f.ShowAllUsers(config, db, sessions, "delete"))
	mux.HandleFunc("GET /user/show-all/edit", f.ShowAllUsers(config, db, sessions, "edit"))

	mux.HandleFunc("POST /article/resubmit/{id}", f.ResubmitArticle(config, db, sessions))
	mux.HandleFunc("POST /article/submit", f.SubmitArticle(config, db, sessions))
	mux.HandleFunc("POST /article/upload-banner", f.UploadImage(config, sessions, "article-banner", "editor.html", "article-banner-template"))
	mux.HandleFunc("POST /article/upload-image", f.UploadArticleImage(config, sessions))
	mux.HandleFunc("POST /docx/upload", f.UploadDocx(config, db, sessions))
	mux.HandleFunc("POST /issue/publish", f.PublishLatestIssue(config, db, sessions))
	mux.HandleFunc("POST /issue/upload-banner", f.UploadImage(config, sessions, "issue-banner", "current-issue.html", "issue-banner-template"))
	mux.HandleFunc("POST /login", f.Login(config, db, sessions, sessionExpiryChan))
	mux.HandleFunc("POST /pdf/upload", f.UploadPDF(config, sessions))
	mux.HandleFunc("POST /tag/add", f.AddTag(config, db, sessions))
	mux.HandleFunc("POST /user/add", f.AddUser(config, db, sessions))
	mux.HandleFunc("POST /user/add-first", f.AddFirstUser(config, db, sessions, sessionExpiryChan))
	mux.HandleFunc("POST /user/update/{id}", f.UpdateUser(config, db, sessions))
	mux.HandleFunc("POST /user/update/self", f.UpdateSelf(config, db, sessions))
	mux.HandleFunc("POST /user/upload-profile-pic", f.UploadImage(config, sessions, "upload-profile-pic", "edit-user.html", "profile-pic-template"))

	log.Fatalln(http.ListenAndServe(config.Port, mux))
}
